/// \brief      handler class implementation
///
/// \date       January 12, 2023
/// \copyright  Costar Technologies, Inc.

#include "handler.hpp"
#include "common/common.hpp"

#include "session.hpp"

handler::handler ( )
{
}

handler::~handler ( )
{
}

void handler::AddSession ( const unsigned short &port, session* pSession )
{
    std::string threadId = boost::lexical_cast<std::string>( boost::this_thread::get_id ( ) );
    lock_stream ( );
    LOG_ERR ( ) << "ThreadId : " << threadId << std::endl;
    LOG_ERR ( ) << "Port : " << port << std::endl;
    unlock_stream ( );
    try
    {
        m_mutex.lock ( );
        m_sessionMap [ port ] = pSession;
        m_mutex.unlock ( );
    }
    catch ( std::exception &exp )
    {
        lock_stream ( );
        LOG_INF ( ) << " Exception " << exp.what ( ) << std::endl;
        unlock_stream ( );
    }
}

void handler::RemoveSession ( const unsigned short &port )
{
    std::string threadId = boost::lexical_cast<std::string>( boost::this_thread::get_id ( ) );
    lock_stream ( );
    LOG_ERR ( ) << "ThreadId : " << threadId << std::endl;
    LOG_ERR ( ) << "Port : " << port << std::endl;
    unlock_stream ( );
    try
    {
        m_mutex.lock ( );
        m_sessionMap.erase ( port );
        m_mutex.unlock ( );
    }
    catch ( std::exception &exp )
    {
        lock_stream ( );
        LOG_INF ( ) << " Exception " << exp.what ( ) << std::endl;
        unlock_stream ( );
    }
}

session* handler::GetSession ( const unsigned short &port )
{
    try
    {
        std::string threadId = boost::lexical_cast<std::string>( boost::this_thread::get_id ( ) );
        lock_stream ( );
        LOG_ERR ( ) << "ThreadId : " << threadId << std::endl;
        LOG_ERR ( ) << "Port : " << port << std::endl;
        unlock_stream ( );
        m_mutex.lock ( );
        session* pSession  = m_sessionMap.at ( port );
        m_mutex.unlock ( );
        if ( pSession == nullptr )
        {
            lock_stream ( );
            LOG_INF ( ) << " session not found " << std::endl;
            unlock_stream ( );
        }
        else
        {
            lock_stream ( );
            LOG_INF ( ) << " session found port = " << port << std::endl;
            unlock_stream ( );
        }
        return  pSession;
    }
    catch ( std::exception &exp )
    {
        lock_stream ( );
        LOG_INF ( ) << " Exception " << exp.what ( ) << std::endl;
        unlock_stream ( );
        return nullptr;
    }
}



