#include <exception>

#include "common/common.hpp"
#include "session.hpp"
#include "packet.hpp"
#include "common/utilities.hpp"

session::session ( shared_ptr<io_service> ios, ReceiveCallBackType pReceiveFn ) noexcept
: m_io_service { ios }
,m_socket {*m_io_service }
,m_packet ( pReceiveFn )
{
    lock_stream ( );
    LOG_ERR ( ) << "Constructor" << std::endl;
    unlock_stream ( );
}

session::~session () noexcept
{
    lock_stream ( );
    LOG_ERR ( ) << "Destructor" << std::endl;
    unlock_stream ( );   
}
tcp::socket &session::get_socket ( ) noexcept
{
    return m_socket;
}

void session::start ( ) noexcept
{
    try
    {
        std::string threadId = boost::lexical_cast<std::string>(boost::this_thread::get_id());
        lock_stream ( );
        LOG_ERR() << "ThreadId : " << threadId << std::endl;
        unlock_stream ( );
        m_handler->AddSession ( get_peer_port(m_socket) , this );
        m_packet.process ( m_socket );
        stop ( );
    }
    catch ( const std::exception &e )
    {
        lock_stream ( );
        LOG_ERR ( ) << "Exception: " << e.what ( ) << std::endl;
        unlock_stream ( );
    }
}

void session::stop ( ) noexcept
{
    unsigned short port = get_peer_port(m_socket);
    lock_stream ( );
    LOG_INF() << "Connection closed with client! ["
                  << get_peer_ip(m_socket) << ":"
                  << port << "]" << std::endl;  
    unlock_stream ( );
    
    m_handler->RemoveSession (port);
    m_socket.shutdown ( boost::asio::socket_base::shutdown_both );
    m_socket.close ( );
}


void session::send ()
{
    m_packet.send (m_socket );
}


void session::setHandler ( shared_ptr<handler> pHandler )
{
   m_handler =  pHandler;
}